@@ -1,15 +1,14 @@
 #include <stdio.h>
 #include <string.h>
+#include <stdint.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <unistd.h>
-int main(int argc, char *argv[]) {
 
-  char    buf[1024];
-  ssize_t i;
-  if ((i = read(0, buf, sizeof(buf) - 1)) < 24) return 0;
-  buf[i] = 0;
+int LLVMFuzzerTestOneInput(const uint8_t *buf, size_t i) {
+
+  if (i < 24) return 0;
   if (buf[0] != 'A') return 0;
   if (buf[1] != 'B') return 0;
   if (buf[2] != 'C') return 0;
@@ -21,3 +20,22 @@ int main(int argc, char *argv[]) {
 
 }
 
+#ifdef __AFL_COMPILER
+int main(int argc, char *argv[]) {
+
+  unsigned char buf[1024];
+  ssize_t       i;
+  while (__AFL_LOOP(1000)) {
+
+    i = read(0, (char *)buf, sizeof(buf) - 1);
+    if (i > 0) buf[i] = 0;
+    LLVMFuzzerTestOneInput(buf, i);
+
+  }
+
+  return 0;
+
+}
+
+#endif
+
